@@ -20,12 +20,19 @@ def toCsv(msg, nameColumn=True, timeColumn=False):
         ret += msg.MsgName() + ", "
     for fieldInfo in Messaging.MsgClass(msg.hdr).fields:
         if(fieldInfo.count == 1):
-            columnText = add_param(str(Messaging.get(msg, fieldInfo)))
-            for bitInfo in fieldInfo.bitfieldInfo:
-                columnText += add_param(str(Messaging.get(msg, bitInfo)))
+            if not fieldInfo.exists(msg):
+                break
+            if len(fieldInfo.bitfieldInfo) == 0:
+                columnText = add_param(str(Messaging.get(msg, fieldInfo)))
+            else:
+                columnText = ""
+                for bitInfo in fieldInfo.bitfieldInfo:
+                    columnText += add_param(str(Messaging.get(msg, bitInfo)))
         else:
             columnText = ""
             for i in range(0,fieldInfo.count):
+                if not fieldInfo.exists(msg, i):
+                    break
                 columnText += add_param(str(Messaging.get(msg, fieldInfo, i)))
         ret += columnText
     # drop last two chars (', ') off end
@@ -43,9 +50,11 @@ def csvHeader(msg, nameColumn=True, timeColumn=False):
     if nameColumn:
         tableHeader += msg.MsgName() + ", "
     for fieldInfo in type(msg).fields:
-        tableHeader += fieldInfo.name + ", "
-        for bitInfo in fieldInfo.bitfieldInfo:
-            tableHeader += bitInfo.name + ", "
+        if len(fieldInfo.bitfieldInfo) == 0:
+            tableHeader += fieldInfo.name + ", "
+        else:
+            for bitInfo in fieldInfo.bitfieldInfo:
+                tableHeader += bitInfo.name + ", "
     # drop last two chars (', ') off end
     return tableHeader[:-2]
 
@@ -96,37 +105,49 @@ def csvToMsg(lineOfText):
                     val = params[paramNumber].strip()
                     #print("val is [" + val + "]") 
                     if(fieldInfo.count == 1):
-                        if val.endswith(";"):
-                            terminateMsg = 1
-                            val = val[:-1]
-                            if val == "":
-                                # terminate without this field
-                                terminationLen = int(fieldInfo.get.offset)
-                                break
-                            # terminate after this field
-                            terminationLen = int(fieldInfo.get.offset) + int(fieldInfo.get.size)
                         if len(fieldInfo.bitfieldInfo) == 0:
+                            if val.endswith(";"):
+                                terminateMsg = 1
+                                val = val[:-1]
+                                if val == "":
+                                    # terminate without this field
+                                    terminationLen = fieldInfo.offset
+                                    break
+                                # terminate after this field
+                                #TODO Broken for arrays-of-structs acting like parallel arrays!
+                                terminationLen = fieldInfo.end_location()
                             if fieldInfo.type == "string":
                                 if val.startswith('"') and val.endswith('"'):
                                     val = val.strip('"')
                                 if terminateMsg:
-                                    terminationLen = int(fieldInfo.get.offset) + int(fieldInfo.get.size) * len(val)
+                                    #TODO Broken for arrays-of-structs acting like parallel arrays!
+                                    terminationLen = fieldInfo.end_location(len(val)-1)
                             Messaging.set(msg, fieldInfo, val)
                             paramNumber+=1
                         else:
                             for bitInfo in fieldInfo.bitfieldInfo:
+                                if val.endswith(";"):
+                                    terminateMsg = 1
+                                    val = val[:-1]
+                                    # terminate without anything after our parent field
+                                    terminationLen = fieldInfo.end_location()
+                                    if val == "":
+                                        break
                                 Messaging.set(msg, bitInfo, val)
                                 paramNumber+=1
                                 val = params[paramNumber]
+                                if terminateMsg:
+                                    break
                     else:
-                        if val.startswith("0x") and len(val) > 2+fieldInfo.count*int(fieldInfo.get.size):
+                        if val.startswith("0x") and len(val) > 2+2*fieldInfo.size:
                             if val.endswith(";"):
                                 terminateMsg = 1
                                 val = val[:-1]
                                 hexStr = val[2:].strip()
-                                terminationLen = int(int(fieldInfo.get.offset) + len(hexStr)/2)
+                                #TODO Broken for arrays-of-structs acting like parallel arrays!
+                                terminationLen = int(fieldInfo.offset + len(hexStr)/2)
                             hexStr = val[2:].strip()
-                            charsForOneElem = int(fieldInfo.get.size)*2
+                            charsForOneElem = fieldInfo.size*2
                             valArray = [hexStr[i:i+charsForOneElem] for i in range(0, len(hexStr), charsForOneElem)]
                             for i in range(0,len(valArray)):
                                 Messaging.set(msg, fieldInfo, int(valArray[i], 16), i)
@@ -135,7 +156,8 @@ def csvToMsg(lineOfText):
                             for i in range(0,fieldInfo.count):
                                 if val.endswith(";"):
                                     terminateMsg = 1
-                                    terminationLen = int(fieldInfo.get.offset) + int(fieldInfo.get.size)*(i+1)
+                                    #TODO Broken for arrays-of-structs acting like parallel arrays!
+                                    terminationLen = fieldInfo.end_location(i)
                                     val = val[:-1]
                                 Messaging.set(msg, fieldInfo, val, i)
                                 if terminateMsg:
