@@ -1,14 +1,12 @@
-#!/usr/bin/env python3
+#!/usr/bin/en"v python3
 from PyQt5 import QtCore, QtGui, QtWidgets, QtSerialPort
 from PyQt5.QtCore import QObject
-from PyQt5.QtCore import QDateTime
 from PyQt5.QtSerialPort import QSerialPort
 
 from msgtools.lib.messaging import Messaging
 from msgtools.lib.header_translator import HeaderTranslator
 from msgtools.server import SerialportDialog
 
-import ctypes
 import struct
 import sys
 
@@ -25,6 +23,13 @@ def Crc16(data):
         crc = 0xFFFF & crc
     return crc
 
+def hexbytes(hdr):
+    try:
+        b = hdr.rawBuffer()[:]
+    except:
+        b = hdr
+    return "0x"+":".join("{:02x}".format(c) for c in b)
+
 class BaseSerialConnection(QObject):
     statusUpdate = QtCore.pyqtSignal(str)
     messagereceived = QtCore.pyqtSignal(object)
@@ -68,6 +73,10 @@ class BaseSerialConnection(QObject):
         self.serialPort.readyRead.connect(self.onReadyRead)
         self.name = self.base_name + " " + self.serialPort.portName()
         self.statusLabel.setText(self.name)
+        
+        self.reopenTimer = QtCore.QTimer(self)
+        self.reopenTimer.setInterval(1000)
+        self.reopenTimer.timeout.connect(self._openSerialPort)
 
     def onDisconnected(self):
         self.disconnected.emit(self)
@@ -78,7 +87,11 @@ class BaseSerialConnection(QObject):
         elif error == QSerialPort.ReadError:
             self.statusUpdate.emit("Error, Closing SerialPort on port "+str(self.serialPort.portName()))
             self._closeSerialPort()
-            QtCore.QTimer.singleShot(1000, self._openSerialPort)
+            # Don't restart the timer if it's active.
+            # If we did, then a steady stream of errors would keep resetting
+            # the timer, and it would never go off.
+            if not self.reopenTimer.isActive():
+                self.reopenTimer.start()
         elif error == QSerialPort.ResourceError:
             pass
 
@@ -94,15 +107,19 @@ class BaseSerialConnection(QObject):
         return None
     
     def _openSerialPort(self):
-        if self.serialPort.open(QSerialPort.ReadWrite):
-            self.statusUpdate.emit("Opened SerialPort on port "+str(self.serialPort.portName()))
-            self.openCloseButton.setText("Close")
-            self.settings.setValue("portName", self.serialPort.portName())
-            return True
+        if self.serialPort.isOpen():
+            # if we're already open, make sure we stop retrying to open
+            self.reopenTimer.stop()
         else:
-            self.openCloseButton.setText("Open")
-            QtCore.QTimer.singleShot(1000, self._openSerialPort)
-            return False
+            if self.serialPort.open(QSerialPort.ReadWrite):
+                self.statusUpdate.emit("Opened SerialPort on port "+str(self.serialPort.portName()))
+                self.openCloseButton.setText("Close")
+                self.settings.setValue("portName", self.serialPort.portName())
+                self.reopenTimer.stop()
+                return True
+            else:
+                self.openCloseButton.setText("Open")
+                return False
 
     def _closeSerialPort(self):
         self.serialPort.close()
@@ -135,7 +152,7 @@ class BaseSerialConnection(QObject):
         self.openCloseSwitch()
 
     def gotRxError(self, errType):
-        print("Got rx error " + errType)
+        print("RX " + errType)
         sys.stdout.flush()
 
     def stop(self):
@@ -148,90 +165,69 @@ class SerialConnection(BaseSerialConnection):
 
         self.hdr = hdr
         
-        self.rxBuffer = bytearray()
-        self.gotHeader = 0
-
         self.hdrTranslator = HeaderTranslator(hdr, Messaging.hdr)
 
-        self.serialStartSeqField = Messaging.findFieldInfo(hdr.fields, "StartSequence")
-        if self.serialStartSeqField != None:
+        serialStartSeqField = Messaging.findFieldInfo(hdr.fields, "StartSequence")
+        if serialStartSeqField != None:
             self.startSequence = int(hdr.GetStartSequence.default)
-            self.startSeqSize = int(hdr.GetStartSequence.size)
+        else:
+            self.startSequence = None
         try:
             self.hdrCrcRegion = int(hdr.GetHeaderChecksum.offset)
         except AttributeError:
             self.hdrCrcRegion = None
-        self.tmpRxHdr = ctypes.create_string_buffer(0)
+        self.tmpRxHdr = None
+
+    def headerValid(self, hdr):
+        if self.startSequence != None and hdr.GetStartSequence() != self.startSequence:
+            #self.gotRxError("  HDR SS %s != %s" % (hex(hdr.GetStartSequence()) , hex(self.startSequence)))
+            return False
+        if self.hdrCrcRegion != None:
+            # Stop computing before we reach header checksum location.
+            headerCrc = Crc16(hdr.rawBuffer()[:self.hdrCrcRegion])
+            receivedHeaderCrc = hdr.GetHeaderChecksum()
+            if headerCrc != receivedHeaderCrc:
+                self.gotRxError("HEADER CRC %s != %s for %s" % (hex(headerCrc) , hex(receivedHeaderCrc), hexbytes(hdr)))
+                return False
+        return True
+    
+    def bodyValid(self, hdr, body):
+        if hdr.GetDataLength() != len(body):
+            self.gotRxError("BODY LENGTH %d != %d" % (hdr.GetDataLength() != len(body)))
+            return False
+        if self.hdrCrcRegion != None:
+            bodyCrc = Crc16(body)
+            receivedBodyCrc = hdr.GetBodyChecksum()
+            if receivedBodyCrc != bodyCrc:
+                self.gotRxError("BODY CRC %s != %s for %s" % (hex(receivedBodyCrc), hex(bodyCrc), hexbytes(body)))
+                return False
+        return True
 
     def onReadyRead(self):
         while self.serialPort.bytesAvailable() > 0:
-            if not self.gotHeader:
-                if self.serialStartSeqField != None:
-                    foundStart = 0
-                    # Synchronize on start sequence, if it exists
-                    while self.serialPort.bytesAvailable() > 0 and self.serialPort.bytesAvailable() >= self.startSeqSize:
-                        # peek at start of message.
-                        #if it's start sequence, break.
-                        #else, throw it away and try again.
-                        self.tmpRxHdr = self.serialPort.peek(self.startSeqSize)
-                        serialHdr = self.hdr(self.tmpRxHdr)
-                        startSequence = serialHdr.GetStartSequence()
-                        if startSequence == self.startSequence:
-                            foundStart = 1
-                            break
-                        else:
-                            print("  " + hex(startSequence) + " != " + hex(self.startSequence))
-                            throwAway = self.serialPort.read(1)
-                            #self.gotRxError("START")
-                else:
-                    foundStart = 1
-
-                if foundStart:
-                    if self.serialPort.bytesAvailable() >= self.hdr.SIZE:
-                        self.tmpRxHdr = self.serialPort.read(self.hdr.SIZE)
-                        serialHdr = self.hdr(self.tmpRxHdr)
-                        if self.serialStartSeqField == None or serialHdr.GetStartSequence() == self.startSequence:
-                            if self.hdrCrcRegion != None:
-                                # Stop counting before we reach header checksum location.
-                                headerCrc = Crc16(self.tmpRxHdr[:self.hdrCrcRegion])
-                                receivedHeaderCrc = serialHdr.GetHeaderChecksum()
-                                if headerCrc == receivedHeaderCrc:
-                                    self.gotHeader = 1
-                                else:
-                                    #self.gotRxError("HEADER")
-                                    print("  " + hex(headerCrc) + " != " + hex(receivedHeaderCrc))
-                            else:
-                                self.gotHeader = 1
-                        else:
-                            self.gotRxError("START")
-                            print("Error in serial parser.  Thought I had start sequence, now it's gone!")
-                    else:
+            if not self.tmpRxHdr:
+                if self.serialPort.bytesAvailable() < self.hdr.SIZE:
+                    return
+                while self.serialPort.bytesAvailable() >= self.hdr.SIZE:
+                    rx_bytes = bytes(self.serialPort.peek(self.hdr.SIZE))
+                    serialHdr = self.hdr(rx_bytes)
+                    if self.headerValid(serialHdr):
+                        # read now, because we only peeked before.
+                        self.tmpRxHdr = self.hdr(self.serialPort.read(self.hdr.SIZE))
                         break
-                else:
-                    break
-
-            if self.gotHeader:
-                serialHdr = self.hdr(self.tmpRxHdr)
-                if self.serialPort.bytesAvailable() >= serialHdr.GetDataLength():
-                    # allocate the serial message body, read from the serial port
-                    bodylen = serialHdr.GetDataLength()
-                    msgBody = self.serialPort.read(bodylen);
-
-                    if self.hdrCrcRegion != None:
-                        bodyCrc = Crc16(msgBody)
-                        receivedBodyCrc = serialHdr.GetBodyChecksum()
-
-                        if receivedBodyCrc != bodyCrc:
-                            self.gotHeader = 0
-                            self.gotRxError("BODY")
-                        else:
-                            self.gotHeader = 0
-                            self.rxMsgCount+=1
-                            self.SerialMsgSlot(serialHdr, msgBody)
                     else:
-                        self.gotHeader = 0
+                        throwAway = self.serialPort.read(1)
+
+            if self.tmpRxHdr != None:
+                bodylen = self.tmpRxHdr.GetDataLength()
+                if self.serialPort.bytesAvailable() >= bodylen:
+                    # allocate the serial message body, read from the serial port
+                    msgBody = self.serialPort.read(bodylen)
+                    
+                    if self.bodyValid(self.tmpRxHdr, msgBody):
                         self.rxMsgCount+=1
-                        self.SerialMsgSlot(serialHdr, msgBody)
+                        self.SerialMsgSlot(self.tmpRxHdr, msgBody)
+                    self.tmpRxHdr = None
                 else:
                     break
 
@@ -255,6 +251,25 @@ def PluginConnection(param=None):
     from SerialHeader import SerialHeader
     return SerialConnection(SerialHeader, param)
 
+def PluginEnabled():
+    try:
+        from SerialHeader import SerialHeader
+        return True
+    except:
+        return False
+
 def BtPluginConnection(param=None):
     from BluetoothHeader import BluetoothHeader
     return SerialConnection(BluetoothHeader, param)
+
+def BtPluginEnabled():
+    try:
+        from BluetoothHeader import BluetoothHeader
+        return True
+    except:
+        return False
+
+import collections
+PluginInfo = collections.namedtuple('PluginInfo', ['name', 'enabled', 'connect_function'])
+plugin_info = PluginInfo('Serial', PluginEnabled, PluginConnection)
+bt_plugin_info = PluginInfo('Bluetooth Serial', BtPluginEnabled, BtPluginConnection)
